//************************************************************
// NUOVO CORSO PROGRAMMAZIONE MICROCONTROLLORI PIC IN C
// (C) Bernardo Giovanni
// https://www.settorezero.com
//
// Tutti i diritti riservati, vietata la
// ridistribuzione e/o riutilizzo anche parziale
// del codice senza consenso scritto dell'autore.
// Al seguente codice si applicano tutte le regole gi
// esposte nell'informativa all'indirizzo:
// https://www.settorezero.com/wordpress/info/termini-di-utilizzo-e-privacy/
//
//************************************************************
// ESEMPIO 6B
// Utilizzo modulo ADCC
// realizzazione di un termometro con display LCD HD44780
//************************************************************
//
// modulo:      main.c
// autore:      Bernardo Giovanni
// mcu:         PIC16F18887
// clock:      	oscillatore interno 16MHz
// compiler:    Microchip XC8 (1.45)
// ide:         MPLAB X (4.15)
// programmer:  PICkit 4
//
//************************************************************

// disabilito i warning per le funzioni non utilizzate
// dato che molte funzioni della libreria LCD non sono usate
// e generano degli innocui warning
#pragma warning disable 520

#include "conf_16F18877.h"
#define _XTAL_FREQ 16000000

#include <xc.h>
#include <stdint.h>
#include <stdbool.h>
#include "settings.h"
#include "hd44780sz.h"

uint16_t temperature=0; // valore di temperatura moltiplicato 10
uint8_t sign=0; // segno temperatura
    
// prototipi funzione
void readAnalog(uint8_t channel);

void main(void)
    {
    SYSTEM_init();
    LCDInit();
    LCDPuts("Test MCP9700");

    while(1)
        {
        readAnalog(0); // leggo dal canale AN0
        LCDGoto(3,1);
        sign?LCDPutch('-'):LCDPutch('+'); //segno
        LCDPutun(temperature/10); // parte intera temperatura
        LCDPutch('.');
        LCDPutun(temperature%10); // decimale
        LCDPutch(223); // simbolo del grado
        LCDPutch('C');
        LCDPuts("   ");
        __delay_ms(250);
        __delay_ms(250);
        }// fine ciclo infinito
    }// fine main

void readAnalog(uint8_t channel)
    {
    int32_t temp=0; // valore temporaneo di temperatura
    //int32_t ADaccumulator=0; // valore nel registro ADACC
    uint8_t i; // counter
    ADPCH=channel;
    // eseguo 16 burst per avere una lettura a display pi stabile
    for (i=0; i<16; i++)
        {
        // avvio la conversione. Da impostazioni del modulo ADCC
        // ogni trigger eseguir 32 misurazioni
        ADCON0bits.ADGO=1;
        // apetto che termina il burst
        while (ADCON0bits.ADGO) {continue;}
		// ADaccumulator=(ADACCH<<8)+ADACCL; // valore del registro ADACC
        // temp += (ADaccumulator>>5); // media fatta a mano
        // in ADFLTR  contenuta la media delle 32 acquisizioni in modalit burst
		temp += ((int32_t)(ADFLTRH<<8)+ADFLTRL);
        // temp+=(uint16_t)((ADRESH<<8)+ADRESL); // lettura da usare solo in modalit legacy
        __delay_ms(10);
        } // ciclo ripetuto 16 volte
	temp >>= 4; // shift destro di 4 posizioni = divisione per 16

	// il modulo ADCC  a 10 bit e viene utilizzato come riferimento l'FVR impostato a 1024mV
	// questo vuol dire che ogni bit vale 1mV (1024mV/2^10)=1mV/Bit
	// questo sensore di temperatura (l'MCP9700) ha questi parametri:
	// range di temperatura: -10 / +125C
	// risoluzione: 10mV/C
	// tensione restituita a 0C = 500mV
	// qui devo convertire il valore in bit restituito dal modulo AD in valore di temperatura espresso in C
	// dalla lettura sottraggo 500mV per riportarmi a zero
    sign=false;
    temp -= 500U;
    if (temp<0)
        {
        sign=true;
        temp *= (-1);
        }

    // dovrei dividere per 10mV/C / 1mV/Bit = 10bit/C
    // ma non lo faccio per non perdere il decimale
    temperature=(unsigned)temp;
    }