#include <xc.h>
#include "settings.h"

// inizializza il sistema
void SYSTEM_init(void)
    {
    IO_init();
    OSC_init();    
    FVR_init();
    ADCC_init();
    }

// inizializza il modulo ADCC
void ADCC_init(void)
    {
    // soglia e setpoint non utilizzati
    ADLTHL=0;
    ADLTHH=0;
    ADUTHL=0;
    ADUTHH=0;
    ADSTPTL=0;
    ADSTPTH=0;
    
    // funzionalit CVD non utilizzate
    ADACQ=0; // no acquisition time
    ADPRE=0; // no precharge
    ADCAP=0; // condensatore aggiuntivo disabilitato

    // 32 ripetizioni
    ADRPT=32;
      
    ADREFbits.ADNREF=0; // riferimento negativo: Vss
    ADREFbits.ADPREF=0b11; // riferimento positivo: valore da modulo FVR

    ADACT=0; // autoconversione disabilitata

    // ADCLK ha solo 5 bit significativi, il valore del clock di conversione
    //  uguale ad FOSC/(ADCLK<<1 + 0x01)
    ADCLK=0x30; // clock di conversione pari a FOSC/97 (Tad=6uS)

    // registri ADCON
    // ADCON3
    ADCON3bits.ADCALC=0; // nel registro ADERR verr memorizzato ADRES-ADREPV
    ADCON3bits.ADSOI=0; // ADGO da resettare a mano piuttosto che in automatico in base alle soglie, ignorato se ADCONT=0
    ADCON3bits.ADTMD=0; // ADIF (interrupt di soglia) disabilitato
    // ADCON2
    ADCON2bits.ADPSIS=1; // nel registro ADPREV trasferiamo ADFLTR
    ADCON2bits.ADCRS=5; // il valore nell'accumulatore sar shiftato a destra di 5 posizioni (diviso per 32)
    ADCON2bits.ADACLR=0; // azzera accumulatore
    ADCON2bits.ADMD=0b011; // burst average mode
    // ADCON1
    ADCON1=0; // una sola conversione (o burst) per trigger, il resto disattivato
    // ADCON0
    ADCON0bits.ADON=1; // modulo attivo
    ADCON0bits.ADCONT=0; // ADGO resettato alla fine della conversione
    ADCON0bits.ADCS=1; // sorgente clock di conversione (1=ADLCK, 0=FRC)
    ADCON0bits.ADFRM0=1; // risultato giustificato a destra (per le modalit 'nuove' non serve)
    ADCON0bits.ADGO=0; // conversione non partita
    }

// Inizializza il modulo Fixed Voltage Reference
void FVR_init(void)
    {
    FVRCONbits.FVREN=1; // modulo FVR abilitato
    FVRCONbits.TSEN=0; // modulo Temperature Indicator disattivato
    FVRCONbits.CDAFVR=0; // modulo FVR non attivo per comparatore (inutile qui)
    FVRCONbits.ADFVR=0b01; // modulo FVR attivo per modulo AD con guadagno 1x (1024mV)
    // aspetto la stabilizzazione del modulo
    while (!FVRCONbits.FVRRDY);
    }

// Inizializza l'oscillatore
void OSC_init(void)
    {
    // L'oscillatore parte a 32MHz come da config word (#pragma config RSTOSC = HFINT32)
    // lo voglio cambiare a 16MHz
    OSCCON1bits.NDIV=0; // divisione per 1
    OSCFRQbits.HFFRQ=0b101; // 16MHz
    while (!OSCSTATbits.HFOR); // attendo che sia pronto
    }

// Inizializza gli IO
void IO_init(void)
    {
    // funzione analogica attiva solo su RA0
    // sui nuovi PICmicro la funzione analogica  attiva su tutti i pin, di tutti i banchi
	ANSELA=0b00000001; 
    ANSELB=0;
	ANSELC=0;
	ANSELD=0;
	ANSELE=0;
    
    // resistenze di pull-up
	WPUA=0;
	WPUB=0;
	WPUC=0;
	WPUD=0;
	WPUE=0;
    
    // imposto tutti i pin come ingresso, la libreria dell'LCD
    // far tutto da se una volta impostato il file H relativo
    // in particolare, gli IO da utilizzare come ingressi analogici vanno
    // sempre configurati come ingressi, in questo caso RA0
    TRISA=0xFF;
    TRISB=0xFF;
    TRISC=0xFF;
    TRISD=0xFF;
    TRISE=0xFF;
    }